import { Patient, Study, Series, Image, Exam } from '../dao'
import { patientTag, studyTag, seriesTag, imageTag } from '../service/dicomTag'
import { uuid16, formatTime } from '../utils/common'
/**
 * 保存dicom信息到pacsonline数据库
 * @param {object} dataSet dicom数据
 * @param {integer} size 影像大小
 * @param {string} fileId fastdfs存储路径
 * @param {string} hospitalId 医院id
 * @param {boolean} isWAN 是否外网路径
 */
const saveDicomInfo2PacsonlineByFileUpload = async(dataSet, size, url, metadata, hospitalId, isWAN) => {
    // console.log(dataSet.string(imageTag.imageId), 'dataSet.string(imageTag.imageId)')
    // return
    const patient = await _savePatient(dataSet, hospitalId) || {}
    const study = await _saveStudy(dataSet, hospitalId, patient.id) || {}
    const serie = await _saveSeries(dataSet, study.id) || {}
    const image = await _saveImage(dataSet, hospitalId, serie.id, url, metadata, size, isWAN) || {}
    const partExamined = dataSet.string(studyTag.partExamined)
    const exam = await _saveExam(hospitalId, patient.id, study.id, patient.temp_patient_id, study.accession_num, study.studydate, study.modality, patient.institution_name, partExamined) || {}
    return {
      patient: patient.id ? true : false,
      study: study.id ? true : false,
      serie: serie.id ? true : false,
      image: image.id ? true : false,
      exam: exam.id ? true : false,
    }
  }
  /**
   * 保存患者信息
   * @param {object} dataSet dicom数据
   * @param {string} hospitalId 医院id
   * */
const _savePatient = async(dataSet, hospitalId) => {
    const temp_patient_id = dataSet.string(patientTag.patientId)
    const where = { temp_patient_id }
      // const where = {temp_patient_id, 'institution_id': hospitalId}
    console.log('where', where)
    let old = await Patient.findOne({ where })
    if (old) {
      return old
    }
    const name = dataSet.string(patientTag.patientName)
    const sex = dataSet.string(patientTag.patientSex)
    const birthday = dataSet.string(patientTag.birthDate)
    const age = dataSet.string(patientTag.patientAge)
    const patient = { 'id': uuid16(), 'institution_id': hospitalId, 'status': 1, temp_patient_id, name, sex, birthday, age }
    try {
      old = await Patient.create(patient);
    } catch (e) {
      old = await Patient.findOne({ where });
    }
    console.log('old', old)
    return old
  }
  /** 
   * 保存检查信息 
   * @param {object} dataSet dicom数据
   * @param {string} hospitalId 医院id
   * @param {string} patient_key 患者key
   * */
const _saveStudy = async(dataSet, hospitalId, patient_key) => {
  const ignoreModalities = ['SR', 'PR']
  const studyuid = dataSet.string(studyTag.studyUid)
  const modality = dataSet.string(studyTag.modalities) || dataSet.string(seriesTag.modality)
  const where = { studyuid, 'patient_id': patient_key }
    // const where             = {studyuid, 'patient_id': patient_key, 'institution_id': hospitalId}
  let old = await Study.findOne({ where })
  let studydate = dataSet.string(studyTag.studyDate)
  if (!studydate || (studydate + '').length < 8) {
    studydate = formatTime()
  }
  if (old) {
    if (old.studydate !== studydate) {
      old.studydate = studydate
    }
    if (old.modality !== modality && ignoreModalities.indexOf(modality) == -1) {
      old.modality = modality
    }
    await old.save();
    // if (modality && ignoreModalities.indexOf(old.modality) != -1 && ignoreModalities.indexOf(modality) == -1) {
    //   old.modality = modality
    //   await old.save()
    // }
    return old;
  }
  const studyid = dataSet.string(studyTag.studyId)
  const accession_num = dataSet.string(studyTag.accessionNumber)
  const patient_age = dataSet.string(studyTag.patientAge)
  const institution_name = dataSet.string(studyTag.institutionName)
  const study = { 'id': uuid16(), 'patient_id': patient_key, 'status': 1, 'institution_id': hospitalId, studyuid, studyid, studydate, accession_num, modality, patient_age, institution_name }
  try {
    old = await Study.create(study)
  } catch (err) {
    old = await Study.findOne({ where })
    console.error("_saveStudy", err, "study", old)
  }
  return old
}

/** 
 * 保存序列信息 
 * @param {object} dataSet dicom数据
 * @param {string} study_key 检查key
 * */
const _saveSeries = async(dataSet, study_key) => {
  const ignoreModalities = ['SR', 'PR']
  const seriesuid = dataSet.string(seriesTag.seriesUid)
  const modality = dataSet.string(seriesTag.modality)
  const where = { seriesuid, modality, 'study_id': study_key }
  let old = await Series.findOne({ where })
  if (old) {
    if (modality && ignoreModalities.indexOf(old.modality) != -1 && ignoreModalities.indexOf(modality) == -1) {
      old.modality = modality
      await old.save()
    }
    return old;
  }
  const series_num = dataSet.intString(seriesTag.seriesNumber)
  const description = dataSet.string(seriesTag.description)
  const serie = { 'id': uuid16(), 'study_id': study_key, seriesuid, modality, series_num, description }
  try {
    old = await Series.create(serie);
  } catch (err) {
    old = await Series.findOne({ where })
    console.error("_saveSeries", err, "series", old)
  }
  return old
}
const IMAGE_URL_STATUS = {
  LOCAL: 0b01,
  REMOTE: 0b10
}

/** 
 * 保存图像信息 
 * @param {object} dataSet dicom数据
 * @param {string} hospitalId 医院id
 * @param {string} series_key 序列key
 * @param {string} url 存储地址
 * @param {integer} size 文件大小
 * @param {boolean} isWAN 是否是外网地址
 * */
const _saveImage = async(dataSet, hospitalId, series_key, url, metadata, size, isWAN) => {
    const image_id = dataSet.string(imageTag.imageId)
    const where = { image_id, 'series_id': series_key }
    let old = await Image.findOne({ where })
    if (old) {
      return old
    }
    const local_url = isWAN ? null : url
    const remote_url = isWAN ? url : null
    const status = isWAN ? IMAGE_URL_STATUS.REMOTE : IMAGE_URL_STATUS.LOCAL
    const sop_uid = dataSet.string(imageTag.SOPInstanceUID)
    const image_number = dataSet.intString(imageTag.imageNumber)
    const pixe_spacing = dataSet.string(imageTag.pixelSpacing)
    const window_width = dataSet.string(imageTag.windowWidth)
    const windo_center = dataSet.string(imageTag.windowCenter)
    const rows = dataSet.int16(imageTag.rows)
    const columns = dataSet.int16(imageTag.columns)
    const image_position = dataSet.string(imageTag.imagePosition)
    const image_orientation = dataSet.string(imageTag.imageOrientation)
    const owner = dataSet.string(patientTag.patientName)
    const frame = dataSet.string(imageTag.numberOfFrames)
    const cineRate = dataSet.string(imageTag.cineRate)
    const image = { 'id': uuid16(), 'series_id': series_key, 'institution_id': hospitalId, 'size': size, 'url': url, 'metadata': metadata, status, image_id, sop_uid, image_number, pixe_spacing, window_width, windo_center, rows, columns, image_position, image_orientation, owner, frame, local_url, remote_url, cineRate }
    try {
      old = await Image.create(image)
    } catch (err) {
      old = await Image.findOne({ where })
      console.error("_saveImage", err)
    }
    return old
  }
  /** 
   * 保存检查信息 
   * @param {string} hospitalId 医院id
   * @param {string} patient_key 患者key
   * @param {string} study_key 检查key
   * @param {string} temp_patient_id 患者id
   * @param {string} accession_num 检查号病历号
   * @param {string} studydate 检查时间
   * @param {string} modality 检查类型
   * @param {string} institution_name 设备名称
   * @param {string} partExamined 检查部位
   * */
const _saveExam = async(hospitalId, patient_key, study_key, temp_patient_id, accession_num, studydate, modality, institution_name, partExamined) => {
  const where = { study_id: study_key }
  const exam = {
    id: uuid16(),
    patient_id: patient_key,
    study_id: study_key,
    exam_status: 3,
    patient_num: temp_patient_id,
    accession_num: accession_num,
    exam_datetime: studydate,
    exam_class: modality,
    institution_id: hospitalId,
    device: institution_name,
    body_part: partExamined
      // exam_method
  }
  let old = await Exam.findOne({ where })
  if (old) {
    if (old.exam_status < exam.exam_status) {
      old.exam_status = exam.exam_status
    }
    if (old.exam_class != exam.exam_class) {
      old.exam_class = exam.exam_class
    }
    if (old.patient_num != exam.patient_num) {
      old.patient_num = exam.patient_num
    }
    if (old.exam_datetime != exam.exam_datetime) {
      old.exam_datetime = exam.exam_datetime
    }
    if (old.accession_num != exam.accession_num) {
      old.accession_num = exam.accession_num
    }
    if (old.device != exam.device) {
      old.device = exam.device
    }
    if (old.body_part != exam.body_part) {
      old.body_part = exam.body_part
    }
    if (old.status === '0') {
      old.status = 1;
    }
    // old.patient_id      = temp.patient_id
    // old.study_id        = temp.study_id
    // old.exam_status     = exam.exam_status
    // old.patient_num     = temp.patient_num
    // old.accession_num   = temp.accession_num
    // old.exam_datetime   = temp.exam_datetime
    // old.exam_class      = temp.exam_class
    // old.institution_id  = temp.institution_id
    await old.save()
    return old
  }
  try {
    old = await Exam.create(exam);
  } catch (e) {
    old = await Exam.findOne({ where })
  }
  return old
}
export { saveDicomInfo2PacsonlineByFileUpload }
